using System;

namespace AppliedData.IO.CAN
{
	public enum FilterMode
	{
		Dual = 0,
		Single = 1
	}
	/// <summary>
	/// Summary description for AcceptanceFilter.
	/// </summary>
	public class AcceptanceFilter
	{
		/*
		typedef struct __CAN_MSG_FILTER
		{
		BOOL mode  ;		// set to 0 for dual filter mode, or 1 for single
		BYTE code0 ;
		BYTE code1 ;
		BYTE code2 ;
		BYTE code3 ;
		BYTE mask0 ;
		BYTE mask1 ;
		BYTE mask2 ;
		BYTE mask3 ;
		} CAN_MSG_FILTER, *PCAN_MSG_FILTER;
		*/

		private const int STRUCT_LENGTH = 12;
		private const int MODE_OFFSET = 0;
		private const int CODE_OFFSET = 4;
		private const int MASK_OFFSET = 8;

		private byte[] m_data = new byte[STRUCT_LENGTH];
				
		public static implicit operator byte[] (AcceptanceFilter f)
		{
			return f.m_data;
		}

		public static implicit operator AcceptanceFilter(byte[] b)
		{
			return new AcceptanceFilter(b);
		}
		
		private AcceptanceFilter(byte[] data)
		{
			if(data.Length != STRUCT_LENGTH)
				throw new ArgumentOutOfRangeException(string.Format("data must be {0} bytes long", STRUCT_LENGTH));

			Buffer.BlockCopy(data, 0, m_data, 0, data.Length);
		}

		public AcceptanceFilter()
		{
		}

		public FilterMode Mode
		{
			get
			{
				return (FilterMode)BitConverter.ToInt32(m_data, MODE_OFFSET);
			}
			set
			{
				Buffer.BlockCopy(BitConverter.GetBytes((int)value), 0, m_data, MODE_OFFSET, 4);
			}
		}

		public int Code
		{
			get
			{
				return BitConverter.ToInt32(m_data, CODE_OFFSET);
			}
			set
			{
				Buffer.BlockCopy(BitConverter.GetBytes(value), 0, m_data, CODE_OFFSET, 4);
			}
		}

		public int Mask
		{
			get
			{
				return BitConverter.ToInt32(m_data, MASK_OFFSET);
			}
			set
			{
				Buffer.BlockCopy(BitConverter.GetBytes(value), 0, m_data, MASK_OFFSET, 4);
			}
		}
	}
}
