using System;

namespace AppliedData.IO.CAN
{
	[Flags]
	public enum MessageFlag : short
	{
		RemoteTransmissionRequest = (1<<0),
		ExtendedIdentifierFormat = (1<<1)
	}

	/// <summary>
	/// Summary description for Message.
	/// </summary>
	public class Message
	{
		internal const int MESSAGE_LENGTH = 16;
		private const int LENGTH_OFFSET = 0;
		private const int ID_OFFSET = 2;
		private const int FLAGS_OFFSET = 6;
		private const int DATA_OFFSET = 8;

		private byte[] m_data = new byte[MESSAGE_LENGTH];

		public static implicit operator byte[] (Message m)
		{
			return m.m_data;
		}

		public static implicit operator Message(byte[] b)
		{
			return new Message(b);
		}

		private Message(byte[] data)
		{
			if(data.Length != MESSAGE_LENGTH)
				throw new ArgumentOutOfRangeException(string.Format("data must be {0} bytes long", MESSAGE_LENGTH));

			Buffer.BlockCopy(data, 0, m_data, 0, data.Length);
		}

		public Message()
		{
		}

		public short Length
		{
			get
			{
				return BitConverter.ToInt16(m_data, LENGTH_OFFSET);
			}
			set
			{
				Buffer.BlockCopy(BitConverter.GetBytes(value), 0, m_data, LENGTH_OFFSET, 2);
			}
		}

		public int ID
		{
			get
			{
				return BitConverter.ToInt32(m_data, ID_OFFSET);
			}
			set
			{
				Buffer.BlockCopy(BitConverter.GetBytes(value), 0, m_data, ID_OFFSET, 4);
			}
		}

		public MessageFlag Flags
		{
			get
			{
				return (MessageFlag)BitConverter.ToInt16(m_data, FLAGS_OFFSET);
			}
			set
			{
				Buffer.BlockCopy(BitConverter.GetBytes((short)value), 0, m_data, FLAGS_OFFSET, 2);
			}
		}

		public void GetData(out long data)
		{
			data = BitConverter.ToInt64(m_data, DATA_OFFSET);
		}

		public void GetData(out int[] data)
		{
			data = new int[] {
								 BitConverter.ToInt32(m_data, DATA_OFFSET),
								 BitConverter.ToInt32(m_data, DATA_OFFSET + 4)
							 };
		}

		public void GetData(out short[] data)
		{
			data = new short[] {
								   BitConverter.ToInt16(m_data, DATA_OFFSET),
								   BitConverter.ToInt16(m_data, DATA_OFFSET + 2),
								   BitConverter.ToInt16(m_data, DATA_OFFSET + 4),
								   BitConverter.ToInt16(m_data, DATA_OFFSET + 6)
							   };
		}

		public void GetData(out byte[] data)
		{
			data = new byte[8];
			Buffer.BlockCopy(m_data, DATA_OFFSET, data, 0, data.Length);
		}
	}
}

