using System;
using System.Data;
using System.Runtime.InteropServices;

namespace AppliedData.IO
{
	/// <summary>
	/// Summary description for SmartIO.
	/// </summary>
	public class SmartIO
	{
		public enum PortName
		{
			A = 0,
			B = 1,
			C = 2,
			D = 3
		}

		#region SMARTIO.DLL Function Imports
		//---------- SMARTIO DEFINITIONS ----------

		[DllImport("smartio.dll", EntryPoint="SIOFirmwareVersion" )]
		private static extern uint SIOFirmwareVersionS();

		[DllImport("smartio.dll", EntryPoint="SIOReadDeviceVersion" )]
		private static extern uint SIOReadDeviceVersionS();

		[DllImport("smartio.dll", EntryPoint="SIOReadDevicePartType" )]
		private static extern uint SIOReadDevicePartTypeS();

		[DllImport("smartio.dll", EntryPoint="SIOReadKeypad" )]
		private static extern byte SIOReadKeypadS();

		[DllImport("smartio.dll", EntryPoint="SIOSetKeyPadSize" )]
		private static extern bool SIOSetKeyPadSizeS(byte cols, byte rows);

		[DllImport("smartio.dll", EntryPoint="SIOSetSpeaker" )]
		private static extern bool SIOSetSpeakerS(byte duration, byte tone);

		[DllImport("smartio.dll", EntryPoint="SIOAnalogConversion" )]
		private static extern uint SIOAnalogConversionS(byte channel);

		[DllImport("smartio.dll", EntryPoint="SIOReadVoltage" )]
		private static extern uint SIOReadVoltageS();

		[DllImport("smartio.dll", EntryPoint="SIOReadTemperature" )]
		private static extern uint SIOReadTemperatureS();

		[DllImport("smartio.dll", EntryPoint="SIOSelectOption" )]
		private static extern void SIOSelectOptionS(byte option);

		[DllImport("smartio.dll", EntryPoint="SIOBackLightPwm" )]
		private static extern void SIOBackLightPwmS(byte val);

		[DllImport("smartio.dll", EntryPoint="SIOBackLightVEEPwm" )]
		private static extern void SIOBackLightVEEPwmS(byte val);

		[DllImport("smartio.dll", EntryPoint="SIOBackLightON" )]
		private static extern void SIOBackLightONS();

		[DllImport("smartio.dll", EntryPoint="SIOBackLightOFF" )]
		private static extern void SIOBackLightOFFS();

		[DllImport("smartio.dll", EntryPoint="SIOWritePortA" )]
		private static extern void SIOWritePortAS(byte val);

		[DllImport("smartio.dll", EntryPoint="SIOWritePortB" )]
		private static extern void SIOWritePortBS(byte val);

		[DllImport("smartio.dll", EntryPoint="SIOWritePortC" )]
		private static extern void SIOWritePortCS(byte val);

		[DllImport("smartio.dll", EntryPoint="SIOWritePortD" )]
		private static extern void SIOWritePortDS(byte val);

		[DllImport("smartio.dll", EntryPoint="SIOReadPortA" )]
		private static extern uint SIOReadPortAS();

		[DllImport("smartio.dll", EntryPoint="SIOReadPortB" )]
		private static extern uint SIOReadPortBS();

		[DllImport("smartio.dll", EntryPoint="SIOReadPortC" )]
		private static extern uint SIOReadPortCS();

		[DllImport("smartio.dll", EntryPoint="SIOReadPortD" )]
		private static extern uint SIOReadPortDS();

		[DllImport("smartio.dll", EntryPoint="SIOInitDDRA" )]
		private static extern void SIOInitDDRAS(byte val);

		[DllImport("smartio.dll", EntryPoint="SIOInitDDRB" )]
		private static extern void SIOInitDDRBS(byte val);

		[DllImport("smartio.dll", EntryPoint="SIOInitDDRC" )]
		private static extern void SIOInitDDRCS(byte val);

		[DllImport("smartio.dll", EntryPoint="SIOInitDDRD" )]
		private static extern void SIOInitDDRDS(byte val);

		[DllImport("smartio.dll", EntryPoint="SIOSetSSPTimeout" )]
		private static extern bool SIOSetSSPTimeoutS(ulong timeout);

		#endregion

		#region KEYBDDR.DLL Function Imports
		//---------- KEYBDDR DEFINITIONS ----------
		
		[DllImport("keybddr.dll", EntryPoint="SIOFirmwareVersion" )]
		private static extern uint SIOFirmwareVersionK();

		[DllImport("keybddr.dll", EntryPoint="SIOReadDeviceVersion" )]
		private static extern uint SIOReadDeviceVersionK();

		[DllImport("keybddr.dll", EntryPoint="SIOReadDevicePartType" )]
		private static extern uint SIOReadDevicePartTypeK();

		[DllImport("keybddr.dll", EntryPoint="SIOReadKeypad" )]
		private static extern byte SIOReadKeypadK();

		[DllImport("keybddr.dll", EntryPoint="SIOSetKeyPadSize" )]
		private static extern bool SIOSetKeyPadSizeK(byte cols, byte rows);

		[DllImport("keybddr.dll", EntryPoint="SIOSetSpeaker" )]
		private static extern bool SIOSetSpeakerK(byte duration, byte tone);

		[DllImport("keybddr.dll", EntryPoint="SIOAnalogConversion" )]
		private static extern uint SIOAnalogConversionK(byte channel);

		[DllImport("keybddr.dll", EntryPoint="SIOReadVoltage" )]
		private static extern uint SIOReadVoltageK();

		[DllImport("keybddr.dll", EntryPoint="SIOReadTemperature" )]
		private static extern uint SIOReadTemperatureK();

		[DllImport("keybddr.dll", EntryPoint="SIOSelectOption" )]
		private static extern void SIOSelectOptionK(byte option);

		[DllImport("keybddr.dll", EntryPoint="SIOBackLightPwm" )]
		private static extern void SIOBackLightPwmK(byte val);

		[DllImport("keybddr.dll", EntryPoint="SIOBackLightVEEPwm" )]
		private static extern void SIOBackLightVEEPwmK(byte val);

		[DllImport("keybddr.dll", EntryPoint="SIOBackLightON" )]
		private static extern void SIOBackLightONK();

		[DllImport("keybddr.dll", EntryPoint="SIOBackLightOFF" )]
		private static extern void SIOBackLightOFFK();

		[DllImport("keybddr.dll", EntryPoint="SIOWritePortA" )]
		private static extern void SIOWritePortAK(byte val);

		[DllImport("keybddr.dll", EntryPoint="SIOWritePortB" )]
		private static extern void SIOWritePortBK(byte val);

		[DllImport("keybddr.dll", EntryPoint="SIOWritePortC" )]
		private static extern void SIOWritePortCK(byte val);

		[DllImport("keybddr.dll", EntryPoint="SIOWritePortD" )]
		private static extern void SIOWritePortDK(byte val);

		[DllImport("keybddr.dll", EntryPoint="SIOReadPortA" )]
		private static extern uint SIOReadPortAK();

		[DllImport("keybddr.dll", EntryPoint="SIOReadPortB" )]
		private static extern uint SIOReadPortBK();

		[DllImport("keybddr.dll", EntryPoint="SIOReadPortC" )]
		private static extern uint SIOReadPortCK();

		[DllImport("keybddr.dll", EntryPoint="SIOReadPortD" )]
		private static extern uint SIOReadPortDK();

		[DllImport("keybddr.dll", EntryPoint="SIOInitDDRA" )]
		private static extern void SIOInitDDRAK(byte val);

		[DllImport("keybddr.dll", EntryPoint="SIOInitDDRB" )]
		private static extern void SIOInitDDRBK(byte val);

		[DllImport("keybddr.dll", EntryPoint="SIOInitDDRC" )]
		private static extern void SIOInitDDRCK(byte val);

		[DllImport("keybddr.dll", EntryPoint="SIOInitDDRD" )]
		private static extern void SIOInitDDRDK(byte val);

		[DllImport("keybddr.dll", EntryPoint="SIOSetSSPTimeout" )]
		private static extern bool SIOSetSSPTimeoutK(ulong timeout);
		
		#endregion

		#region Exposed functions

		// functions try to call from smartio.dll first.  
		// if that fails, they call keybddr.dll
		// if that fails, they throw an exception

        /// <summary>
        /// Gets the Firmware Version number from the ADSmartIO microcontroller
        /// </summary>
        /// <returns>Firmware Version number</returns>
		public static uint FirmwareVersion()
		{
			try
			{
				return SIOFirmwareVersionS();
			}
			catch(Exception)
			{
				try
				{
					return SIOFirmwareVersionK();

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Gets the ASCII character of the revision level of the ADSmartIO
        /// </summary>
        /// <returns>Character of ADSmartIO revision level</returns>
        public static uint DeviceVersion()
		{
			try
			{
				return SIOReadDeviceVersionS();
			}
			catch(Exception)
			{
				try
				{
					return SIOReadDeviceVersionK();

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Gets the ADSmartIO microcontroller part type
        /// </summary>
        /// <returns>ADSmartIO microcontroller part type</returns>
		public static uint DevicePartType()
		{
			try
			{
				return SIOReadDevicePartTypeS();
			}
			catch(Exception)
			{
				try
				{
					return SIOReadDevicePartTypeK();

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Reads the keypad data
        /// </summary>
        /// <returns>Returns keypad Data, 0xff if there is a timeout</returns>
		public static byte ReadKeypad()
		{
			try
			{
				return SIOReadKeypadS();
			}
			catch(Exception)
			{
				try
				{
					return SIOReadKeypadK();

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Sets the size of the keypad
        /// The columns are driven using Port A, and the Rows are sensed using Port C. 
        /// If a keypad matrix is less then 8x8, the unused pins can be used as digital/analog I/Os. 
        /// The Port pins are used starting with bit 0. The Keypad scan will not start until Option 1 is set and
        /// SetKeypadSize are issued in that order. After Option1 is set Ports A and C will
        /// remain as inputs until a value is written with SetKeypadSize. 
        /// To use Option 1 settings but disable keyscan, set row and column sizes to zero.
        /// When a key is pressed, the AVR returns a number between 0 and 63 calculated as follows:
        ///     char = column + (row  1) * 8
        /// With row and column the coordinates of the key pressed (1 to 8). 
        /// See the ReadKeypad() for reading the Keypad Data.
        /// </summary>
        /// <param name="Columns">Columns of the Keypad, between 1 and 8</param>
        /// <param name="Rows">Rows of the Keypad, between 1 and 8</param>
        /// <returns>1 for success, 0 for error</returns>
		public static bool SetKeyPadSize(byte Columns, byte Rows)
		{
			try
			{
				return SIOSetKeyPadSizeS(Columns, Rows);
			}
			catch(Exception)
			{
				try
				{
					return SIOSetKeyPadSizeK(Columns, Rows);

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}
		
        /// <summary>
        /// Set the speaker with duration and tone
        /// </summary>
        /// <param name="Duration">Duration is the hexadecimal value of the duration of the tone</param>
        /// <param name="Tone">Tone is a hexadecimal value proportional to the period (1/frequency) of the tone</param>
        /// <returns>16 bit A/D data, 0xffff if there is a timeout</returns>
		public static bool SetSpeaker(byte Duration, byte Tone)
		{
			try
			{
				return SIOSetSpeakerS(Duration, Tone);
			}
			catch(Exception)
			{
				try
				{
					return SIOSetSpeakerK(Duration, Tone);

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}
	    
        /// <summary>
        /// Read A/D Data from a specified channel
        /// </summary>
        /// <param name="Channel">Channel Number</param>
        /// <returns>10 Bit A/D Data, 0xffff if there is a timeout</returns>
		public static uint AnalogConversion(byte Channel)
		{
			try
			{
				return SIOAnalogConversionS(Channel);
			}
			catch(Exception)
			{
				try
				{
					return SIOAnalogConversionK(Channel);

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Read Voltage Level
        /// </summary>
        /// <returns>16 bit Voltage Level</returns>
		public static uint ReadVoltage()
		{
			try
			{
				return SIOReadVoltageS();
			}
			catch(Exception)
			{
				try
				{
					return SIOReadVoltageK();

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Read Temperature Level
        /// </summary>
        /// <returns>16 bit Voltage Level</returns>
		public static uint ReadTemperature()
		{
			try
			{
				return SIOReadTemperatureS();
			}
			catch(Exception)
			{
				try
				{
					return SIOReadTemperatureK();

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}
		
        /// <summary>
        /// Select One of three ADSmartIO Option configurations.
        /// See the ADSmartIO Driver Spec for more information.
        /// </summary>
        /// <param name="OptionValue">Option 1, 2, or 3</param>
		public static void SelectOption(byte OptionValue)
		{
			try
			{
				SIOSelectOptionS(OptionValue);
			}
			catch(Exception)
			{
				try
				{
					SIOSelectOptionK(OptionValue);

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Set the backlight level
        /// </summary>
        /// <param name="NewValue"> 0xff for Low Intensity, 0 for Maximum Intensity</param>
		public static void BacklightPwm(byte NewValue)
		{
			try
			{
				SIOBackLightPwmS(NewValue);
			}
			catch(Exception)
			{
				try
				{
					SIOBackLightPwmK(NewValue);

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Set Contrast Level
        /// </summary>
        /// <param name="NewValue">0 for Low Contrast, 0xff for Maximum Contrast</param>
		public static void VEEPwm(byte NewValue)
		{
			try
			{
				SIOBackLightVEEPwmS(NewValue);
			}
			catch(Exception)
			{
				try
				{
					SIOBackLightVEEPwmK(NewValue);

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Turn on the backlight
        /// </summary>
		public static void BacklightOn()
		{
			try
			{
				SIOBackLightONS();
			}
			catch(Exception)
			{
				try
				{
					SIOBackLightONK();

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Turn off the backlight
        /// </summary>
		public static void BacklightOff()
		{
			try
			{
				SIOBackLightOFFS();
			}
			catch(Exception)
			{
				try
				{
					SIOBackLightOFFK();

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Set timeout for waiting response from ADSmartIO microcontroller
        /// </summary>
        /// <param name="Timeout">Timeout in milliseconds</param>
        /// <returns>True</returns>
		public static bool SetSSPTimeout(ulong Timeout)
		{
			try
			{
				return SIOSetSSPTimeoutS(Timeout);
			}
			catch(Exception)
			{
				try
				{
					return SIOSetSSPTimeoutK(Timeout);

				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Write a byte of data to the specified port
        /// </summary>
        /// <param name="Port">Port to write data to</param>
        /// <param name="NewValue">Data to write to port</param>
		public static void WritePort(PortName Port, byte NewValue)
		{
			try
			{
				switch(Port)
				{
					case PortName.A:
						SIOWritePortAS(NewValue);
						break;
					case PortName.B:
						SIOWritePortBS(NewValue);
						break;
					case PortName.C:
						SIOWritePortCS(NewValue);
						break;
					case PortName.D:
						SIOWritePortDS(NewValue);
						break;
				}
			}
			catch(Exception)
			{
				try
				{
					switch(Port)
					{
						case PortName.A:
							SIOWritePortAK(NewValue);
							break;
						case PortName.B:
							SIOWritePortBK(NewValue);
							break;
						case PortName.C:
							SIOWritePortCK(NewValue);
							break;
						case PortName.D:
							SIOWritePortDK(NewValue);
							break;
					}
				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Read a byte of data from the specified port
        /// </summary>
        /// <param name="Port">The port to read from</param>
        /// <returns>The data read from the port</returns>
		public static uint ReadPort(PortName Port)
		{
			try
			{
				switch(Port)
				{
					case PortName.A:
						return SIOReadPortAS();
					case PortName.B:
						return SIOReadPortBS();
					case PortName.C:
						return SIOReadPortCS();
					case PortName.D:
						return SIOReadPortDS();
					default:
						return uint.MaxValue;
				}
			}
			catch(Exception)
			{
				try
				{
					switch(Port)
					{
						case PortName.A:
							return SIOReadPortAK();
						case PortName.B:
							return SIOReadPortBK();
						case PortName.C:
							return SIOReadPortCK();
						case PortName.D:
							return SIOReadPortDK();
						default:
							return uint.MaxValue;
					}
				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

        /// <summary>
        /// Set the data direction for the specified port
        /// </summary>
        /// <param name="Port">The port to set the direction</param>
        /// <param name="DDRValue">The direction configuration.
        ///     0 = Input
        ///     1 = Output</param>
		public static void InitDDR(PortName Port, byte DDRValue)
		{
			try
			{
				switch(Port)
				{
					case PortName.A:
						SIOInitDDRAS(DDRValue);
						break;
					case PortName.B:
						SIOInitDDRBS(DDRValue);
						break;
					case PortName.C:
						SIOInitDDRCS(DDRValue);
						break;
					case PortName.D:
						SIOInitDDRDS(DDRValue);
						break;
				}
			}
			catch(Exception)
			{
				try
				{
					switch(Port)
					{
						case PortName.A:
							SIOInitDDRAK(DDRValue);
							break;
						case PortName.B:
							SIOInitDDRBK(DDRValue);
							break;
						case PortName.C:
							SIOInitDDRCK(DDRValue);
							break;
						case PortName.D:
							SIOInitDDRDK(DDRValue);
							break;
					}
				}
				catch(Exception e)
				{
					throw e;
				}
			}
		}

		#endregion
	}
}
